from typing import Any, Callable
from abc import abstractmethod, ABC

from typing import List


# noinspection PyPep8Naming
def IndsType(name: str, super_type: type, constraint: Callable[[Any], bool]):
    def inds_type(x: Any) -> bool:
        return constraint(x)

    inds_type.__name__ = name
    inds_type.__supertype__ = super_type
    return inds_type


from functools import partial


class FilterReference:
    def __init__(self, app: 'Application', filter_name: str):
        self.__callee__: Callable[[], 'LinkFilter'] = partial(app.filters.get, filter_name)

    def __call__(self) -> 'LinkFilter':
        return self.__callee__()


class NodeReference:
    def __init__(self, app: 'Application', node_name: str):
        self.__callee__: Callable[[], 'Node'] = partial(app.nodes.get, node_name)

    def __call__(self) -> 'Node':
        return self.__callee__()


class VariableReference:
    def __init__(self, app: 'Application', var_name: str):
        self.__callee__: Callable[[], Any] = partial(app.getattr, var_name)

    def __call__(self) -> 'Any':
        return self.__callee__()
