from abc import ABC, abstractmethod
from enum import Enum
from typing import Optional

from inds.core.item import Item, List

import numpy as np


class InferenceMode(str, Enum):
    WHOLE_IMAGE = 'whole_image'
    ALL_ROIS = 'all_rois'
    SELECTED_ROIS = 'selected_rois'


class Runtime(str, Enum):
    ONNX_RUNTIME = 'onnxruntime'
    TENSORRT = 'tensorrt'


class BaseModel(ABC):

    @abstractmethod
    def preprocess_item(self, item: Item) -> Item:
        """
        Preprocess an item before anything else happens.
        """
        pass


    @abstractmethod
    def preprocess(self, item: Item) -> Optional[np.ndarray]:
        pass

    @abstractmethod
    def predict(self, input_batch: np.ndarray) -> List[np.ndarray]:
        pass

    @abstractmethod
    def postprocess(self, prediction_output, item: Item):
        pass

    @abstractmethod
    def destroy(self):
        pass
